<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Evenement' => array($vendorDir . '/evenement/evenement/src'),
    'Clue\\Redis\\Protocol' => array($vendorDir . '/clue/redis-protocol/src'),
    'AssetLoader' => array($baseDir . '/'),
);
